local abs = math.abs
local sgn = sgn

local camera = {
	x = 0,
	y = 0,
	scale = 1,

	tx = 0,
	ty = 0,
	tscale = 0,
}

function camera:applyLimits()
	self.x = clamp(self.x, game.world.boundaries.left + 400 / self.scale, game.world.boundaries.right - 400 / self.scale)
	self.y = clamp(self.y, game.world.boundaries.top + 300 / self.scale, game.world.boundaries.bottom - 300 / self.scale)
end

function camera:setPosition(x, y, scale)
	self.x, self.y = x, y
	self.scale = scale
end

function camera:setTarget(x, y, scale)
	self.tx, self.ty = x, y
	self.tscale = scale
end

function camera:setup()
	love.graphics.translate(-self.x * self.scale + 400, -self.y * self.scale + 300)
end

function camera:update(dt)
	local dx, dy = self.tx - self.x, self.ty - self.y
	local ds = self.tscale - self.scale

	if abs(dx) < 0.001 then
		self.x = self.tx
	else
		self.x = self.x + dx * 5 * dt
	end

	if abs(dy) < 0.001 then
		self.y = self.ty
	else
		self.y = self.y + dy * 5 * dt
	end

	if abs(ds) < 0.001 then
		self.scale = self.tscale
	else
		local step = ds * 3 * dt
		self.scale = self.scale + step
	end

	self:applyLimits()
end

return camera
