require("utils")
require("rectanglewithroundcorners")

function love.load()
	love.audio.setVolume(0.5)
	love.physics.setMeter(1)
	love.graphics.setBackgroundColor(60, 200, 255)
	love.graphics.setIcon(love.graphics.newImage("icon.png"))
	game = {}

	game.help = true
	game.text = {
		help = [[
Pigs can't fly! Somehow, this one managed to get up there anyway. It's your job to get it safely to the ground.

The pig switched over to emergency mode and made its thruster avialable to you. Sadly, the fuel lasts for only 8 seconds more. If that's not bad enough, the pig is extremely vulnerable in emergency mode and will be killed if it hits the ground slightly too hard. The thruster will shutdown if the pig touches anything.

It's possible that the pig is over terrain unsuitable for landing.]],
	}

	game.fontextrasmall = love.graphics.newFont(11)
	game.fontverysmall = love.graphics.newFont(12)
	game.fontsmall = love.graphics.newFont("AveriaGruesaLibre-Regular.ttf", 16)
	game.fontmedium = love.graphics.newFont("AveriaGruesaLibre-Regular.ttf", 26)
	game.fontlarge = love.graphics.newFont("AveriaGruesaLibre-Regular.ttf", 40)

	game.keys = {}
	game.keys.F1 = love.graphics.newImage("F1.png")
	game.keys.P = love.graphics.newImage("P.png")
	game.keys.space = love.graphics.newImage("space.png")
	game.keys["return"] = love.graphics.newImage("return.png")
	game.keys.left = love.graphics.newImage("left.png")
	game.keys.up = love.graphics.newImage("up.png")
	game.keys.right = love.graphics.newImage("right.png")

	game.lovelogo = love.graphics.newImage("lovelogo.png")

	game.camera = require("camera")
	game.world = require("world")
	game.world:create()
	game.world:generate()
	setCameraTarget(true)

	game.sound = require("sound")
end

function setCameraTarget(andPos)
	local px, py, gx, gy, diff = game.world:findGround()
	if not game.world.pig.outside then
		local x, y, s = px, py + diff / 2, clamp(300 / diff, 2, 30)
		game.camera:setTarget(x, y, s)
		if andPos then
			game.camera:setPosition(x, y, s)
		end
	end
end

function drawHelp()
	love.graphics.setColor(10, 10, 10, 220)
	rwrc:draw(30, -30, 740, 580, 30)
	love.graphics.setColor(60, 60, 69, 220)
	rwrc:draw(500, 40, 250, 420, 20)

	love.graphics.setColor(255, 160, 209)
	love.graphics.print("Piggy Lander", 50, 20)

	love.graphics.setColor(128, 128, 128)
	love.graphics.setFont(game.fontextrasmall)
	love.graphics.print("freesound.org/people/confusion_music", 515, 368)

	love.graphics.setFont(game.fontverysmall)
	love.graphics.print("Version: 1.bacon", 310, 48)

	love.graphics.setColor(255, 255, 255)
	love.graphics.print("Open / Close", 665, 530)



	love.graphics.setFont(game.fontsmall)
	love.graphics.printf(game.text.help, 55, 80, 400, "left")
	love.graphics.print("Made with", 590, 45)

	love.graphics.print([[
LÖVE uses: FreeType 2, Box2D,
DevIL, FreeType, libogg,
libvorbis, libpng, Lua,
OpenAL (Soft), PhysicsFS, SDL,
UTF8-CPP, zlib.]], 515, 120)

	love.graphics.print([[
Piggy Lander uses:
The Averia Gruesa Libre type
by Dan Sayers.

Several pig sounds
recorded by confusion_music.]], 515, 250)

	love.graphics.setFont(game.fontmedium)
	love.graphics.print("Pig Thruster        Pause", 90, 395)
	love.graphics.print("Reset level              New level", 120, 505)

	love.graphics.draw(game.lovelogo, 560, 55)

	love.graphics.draw(game.keys.left, 60, 325)
	love.graphics.draw(game.keys.up, 60+64, 325)
	love.graphics.draw(game.keys.right, 60+64*2, 325)
	love.graphics.draw(game.keys.P, 290, 325)

	love.graphics.draw(game.keys.space, 60, 435)
	love.graphics.draw(game.keys["return"], 350, 375)

	love.graphics.draw(game.keys.F1, 690, 490, 0, 0.5, 0.5)
end

function love.update(dt)
	game.world:update(dt)

	setCameraTarget()
	game.camera:update(dt)
--	game.camera:setPosition(game.world.pig.b:getX(), game.world.pig.b:getY(), 50)
	game.scale = game.camera.scale
end

function love.draw()
	love.graphics.push()
	game.camera:setup()
	game.world:draw()
	love.graphics.pop()

	love.graphics.setFont(game.fontlarge)

	local timeLeft = string.format("%6.3f", math.max(0, 8 - game.world.timer))
	if game.world.timer >= 8 then
		love.graphics.setColor(255, 0, 0, 128)
	else
		love.graphics.setColor(50, 50, 50, 128)
	end
	rwrc:draw(660, 20, 165, 50, 10)
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(timeLeft, 670, 20)

	if game.world.pig.landed and game.world.timelanded then
		love.graphics.setColor(50, 50, 50, 128)
		rwrc:draw(90, 400, 620, 55, 10)
		love.graphics.setColor(255, 255, 255)
		love.graphics.printf(string.format("The pig landed in %.3f seconds!", game.world.timelanded), 0, 400, 800, "center")
	elseif game.world.pig.dead then
		love.graphics.setColor(50, 50, 50, 128)
		rwrc:draw(250, 400, 300, 55, 10)
		love.graphics.setColor(248, 40, 40)
		love.graphics.printf("The pig is dead!", 0, 400, 800, "center")
		love.graphics.setColor(255, 255, 255)
	end

	if game.help then
		drawHelp()
	else
		if game.world:isPaused() then
			love.graphics.setColor(50, 50, 50, 128)
			rwrc:draw(315, 20, 170, 50, 10)
			love.graphics.setColor(255, 255, 255)
			love.graphics.printf("PAUSED", 0, 20, 800, "center")
		end
	end
end

function love.keypressed(key, unicode)
	if key == "escape" or (love.keyboard.isDown("lalt", "ralt") and key == "f4") then
		love.event.quit()
	elseif key == "f1" then
		game.help = not game.help
	elseif key == "p" or key == "pause" then
		game.world.paused = not game.world.paused
	end
end

function love.keyreleased(key, unicode)
	if key == " " then
		game.world:reset()
		setCameraTarget(true)
	elseif key == "kpenter" or key == "return" then
		game.world:generate()
		setCameraTarget(true)
	end
end

function love.quit()
	game.world:destroy()
end


function love.run()

    math.randomseed(os.time())
    math.random() math.random()

    if love.load then love.load(arg) end

    local dt = 0

    -- Main loop time.
    while true do
		local starttime = love.timer.getTime()
        -- Process events.
        if love.event then
            love.event.pump()
            for e,a,b,c,d in love.event.poll() do
                if e == "quit" then
                    if not love.quit or not love.quit() then
                        if love.audio then
                            love.audio.stop()
                        end
                        return
                    end
                end
                love.handlers[e](a,b,c,d)
            end
        end

        -- Update dt, as we'll be passing it to update
        if love.timer then
            love.timer.step()
            dt = love.timer.getDelta()
        end

        -- Call update and draw
        if love.update then love.update(dt) end -- will pass 0 if love.timer is disabled
        if love.graphics then
            love.graphics.clear()
            if love.draw then love.draw() end
        end

        if love.graphics then love.graphics.present() end

		local endtime = love.timer.getTime()
		if endtime - starttime < 1/60 then
			love.timer.sleep(1/60 - clamp(endtime - starttime, 0, 1/60))
		end
    end

end
