local pig = {}

pig.front = {}
pig.front.img = love.graphics.newImage("pig_front.png")
pig.front.deadeyes = love.graphics.newImage("deadeyes.png")
pig.front.width = pig.front.img:getWidth()
pig.front.height = pig.front.img:getHeight()
pig.front.width2, pig.front.height2 = pig.front.width * 0.5, pig.front.height * 0.5
pig.front.cx, pig.front.cy = 128, 128
pig.front.scale = 256 / 235

pig.side = {}
pig.side.img = love.graphics.newImage("pig_side.png")
pig.side.width = pig.side.img:getWidth()
pig.side.height = pig.side.img:getHeight()
pig.side.width2, pig.side.height2 = pig.side.width * 0.5, pig.side.height * 0.5
pig.side.cx, pig.side.cy = 122, 166
pig.side.scale = 256 / 180

pig.bottom = {}
pig.bottom.img = love.graphics.newImage("pig_bottom.png")
pig.bottom.width = pig.bottom.img:getWidth()
pig.bottom.height = pig.bottom.img:getHeight()
pig.bottom.width2, pig.bottom.height2 = pig.bottom.width * 0.5, pig.bottom.height * 0.5
pig.bottom.cx, pig.bottom.cy = 126, 155
pig.bottom.scale = 256 / 179

function pig:create(world)
	self.bacon = require("bacon")
	self.b = love.physics.newBody(world, 0, 0, "dynamic")

	self.sc = love.physics.newCircleShape(0.75)
	self.sl = love.physics.newRectangleShape(-0.46, 0.65, 0.15, 0.3)
	self.sr = love.physics.newRectangleShape(0.46, 0.65, 0.15, 0.3)

	self.f = {}
	self.f.fc = love.physics.newFixture(self.b, self.sc, 108)
	self.f.fl = love.physics.newFixture(self.b, self.sl, 108)
	self.f.fr = love.physics.newFixture(self.b, self.sr, 108)

	self.f.fc:setGroupIndex(-1)
	self.f.fl:setGroupIndex(-1)
	self.f.fr:setGroupIndex(-1)

	self.f.fc:setFriction(0.25)
	self.f.fl:setFriction(0.25)
	self.f.fr:setFriction(0.25)


	--self.pig.b:setActive(false)
	--error(self.b:getMass())
	--self.pig.b:setFixedRotation(true)
	self.b:setLinearDamping(0.1765)

	self.dropimg = love.graphics.newImage("drop.png")
	self.thruster = love.graphics.newParticleSystem(self.dropimg, 1000)
	self.thruster:setOffset(self.dropimg:getWidth()/2, self.dropimg:getHeight()/2)
	self.thruster:setLifetime(-1)
	self.thruster:setParticleLife(0.05, 0.1)
	self.thruster:setEmissionRate(200)
	self.thruster:setSpeed(30, 50)
	self.thruster:setSizes(0.025, 0.018)
	self.thruster:setColors(255, 255, 255, 180)
	self.thruster:setSizeVariation(0, 1)
	self.thruster:setSpread(0.75)
	self.thruster:setRotation(0, math.pi*2)
	self.thruster:setSpin(3, 6)
end

function pig:draw()
	local x, y = scaler(game.scale, self.b:getPosition())

	local r = self.b:getAngle()
	if not self.exploded then
		if self.dir then
			if self.dir == 0 then
				local sx, sy = 1.5 / self.bottom.width * game.scale * self.bottom.scale, 1.5 / self.bottom.height * game.scale * self.bottom.scale
				love.graphics.draw(self.bottom.img, x, y, 0, sx, sy, self.bottom.cx, self.bottom.cy)
			else
				local sx, sy = 1.5 / self.bottom.width * game.scale * self.bottom.scale, 1.5 / self.bottom.height * game.scale * self.bottom.scale
				love.graphics.draw(self.side.img, x, y, self.dir, sgn(self.dir) * sx, sy, self.side.cx, self.side.cy)
			end
			love.graphics.draw(self.thruster, 0, 0, 0, game.scale, game.scale)
		else
			local sx, sy = 1.5 / self.front.width * game.scale * self.front.scale, 1.5 / self.front.height * game.scale * self.front.scale
			love.graphics.draw(self.front.img, x, y, r, sx, sy, self.front.cx, self.front.cy)
			if self.dead then
				love.graphics.draw(self.front.deadeyes, x, y, r, sx, sy, self.front.cx, self.front.cy)
			end
		end
	end
	self.bacon:draw()
end

function pig:reset()
	self.b:setActive(true)
	self.dead = false
	self.landed = false
	self.exploded = false
	self.outside = false
	self.inControl = true
	self.dir = nil
	self.b:setPosition(pig.initx, pig.inity)
	self.b:setAngle(0)
	self.b:setLinearVelocity(0, 0)
	self.b:setAngularVelocity(0)
	self.b:setAwake(true)
	
	self.f.fl:setSensor(false)
	self.f.fr:setSensor(false)
	self.thruster:reset()
end

function pig:death(hard)
	if not self.dead then
		if not hard then
			game.sound:death()
		else
			game.sound:harddeath()
		end
	end

	self.dead = true

	if hard and not self.exploded then
		self.exploded = true
		self.bacon:explodeAt(self.b:getPosition())
	end

	self.inControl = false
	self.dir, self.dirx, self.diry = nil, nil, nil

	if self.exploded or self.outside then
		self.b:setActive(false)
	end
	self.f.fl:setSensor(true)
	self.f.fr:setSensor(true)
end

function pig:update(dt)
	if self.dir then
		self.thruster:setDirection((self.dir or 0) + math.pi/2)
		self.thruster:setPosition(self.b:getX() - self.dirx * 0.375, self.b:getY() - self.diry * 0.375)
		self.thruster:start()
	else
		self.thruster:pause()
	end
	self.thruster:update(dt)
	self.bacon:update(dt)

	if not self.landed and not self.b:isAwake() and not self.dead then
		self.landed = true
		game.sound:happy()
	end
end

return pig
