local world = {
	boundaries = {
		left = 0,
		top = 0,
		right = 400,
		bottom = 400,
	},

	paused = false,
	timer = 0,
}


local preCallback
do
	local dead, hard
	preCallback = function(f1, f2, con)
		if f1 == nil then
			local a, b = dead, hard
			dead, hard = nil, nil
			return a, b
		end

		local pig = game.world.pig
		if
			f1 == pig.f.fc or f2 == pig.f.fc or
			f1 == pig.f.fl or f2 == pig.f.fl or
			f1 == pig.f.fr or f2 == pig.f.fr
		then
			pig.inControl = false
			pig.dir, pig.dirx, pig.diry = nil, nil, nil
			if not game.world.timelanded then
				game.world.timelanded = game.world.timer
			end

			local vx, vy = pig.b:getLinearVelocity()
			local nx, ny = con:getNormal()
			local velocity = ((vx*nx)^2 + (vy*ny)^2)^0.5
			if velocity >= 5 then
				--print(velocity)
				dead = true
				if velocity >= 10 then
					hard = true
				end
			end
		elseif f1:getUserData() == "poop" then
			local a, b, x, y = con:getPositions()
			game.world.terrain.shit:hit(f1, x, y)
		elseif f2:getUserData() == "poop" then
			local x, y = con:getPositions()
			game.world.terrain.shit:hit(f2, x, y)
		end
	end
end

function world:create()
	self.world = love.physics.newWorld(0, 9.81)
	self.terrain = require("terrain")
	self.pig = require("pig")
	self.pig:create(self.world)

	self.clouds = love.graphics.newImage("clouds.png")

	self.world:setCallbacks(nil, nil, preCallback, nil)
end

function world:cleanup()
	if self.world then
		local bodylist = self.world:getBodyList()
		for i = 1, #bodylist do
			bodylist[i]:destroy()
		end
	end
end

function world:destroy()
	self.world:destroy()
end

function world:isPaused()
	return self.paused or not love.graphics.hasFocus() or game.help
end

function world:generate()
	local pig, tr = self.pig, self.terrain
	if tr.b then
		tr.b:destroy()
		tr.b, tr.s, tr.f = nil, nil, nil
	end

	tr:generate()
	tr.b = love.physics.newBody(self.world, 0, 0, "static")
	tr.s = love.physics.newChainShape(false, unpack(tr.points))
	tr.f = love.physics.newFixture(tr.b, tr.s)

	pig.initx = math.random(150, 250)
	pig.inity = 50

	self:reset()
end

local timer = 0
local step = 1/128
function world:update(dt)
	if self:isPaused() or dt <= 0 or dt > 0.5 then
		return
	end

	local tr, pig = self.terrain, self.pig

	if not pig.dead and not pig.landed then
		if self.timer == 0 then
			self.timer = 1e-100
		else
			self.timer = self.timer + dt
		end
	end

	if self.timer >= 8 then
		pig.inControl = false
		pig.dir, pig.dirx, pig.diry = nil, nil, nil
	end

	if self.timer > 0 and self.timer < 9 and self.beeptimer - self.timer <= 0 then
		game.sound:beep(0.8 + math.floor(self.timer)/8 * 0.4)
		self.beeptimer = self.beeptimer + 1
	end

	if pig.inControl and pig.b:isActive() then
		local vx = (love.keyboard.isDown("left") and -1 or 0) + (love.keyboard.isDown("right") and 1 or 0)
		local vy = love.keyboard.isDown("up") and -1 or 0
		local length = (vx*vx + vy*vy)^0.5

		if length > 0 then
			vx, vy = vx / length, vy / length
			if vx == 0 and vy < 0 then
				pig.dir = 0
			elseif vy == 0 and vx ~= 0 then
				pig.dir = math.pi / 2 * sgn(vx)
			elseif vy ~= 0 and vx ~= 0 then
				pig.dir = math.pi / 4 * sgn(vx)
			end
			pig.dirx, pig.diry = vx, vy
			pig.b:applyForce(vx * 5000, vy * 5000)

			if math.random() > 0.98 then
				tr.shit:addPiece(pig.b:getX(), pig.b:getY(), vx, vy)
			end
		else
			pig.dir, pig.dirx, pig.diry = nil, nil, nil
		end
	end

	timer = timer - dt
	while timer <= 0 do
		self.world:update(step)
		timer = timer + step
	end
	local isDead, isExploded = preCallback()
	if isDead then
		self.pig:death(isExploded)
	end

	local pigx, pigy = pig.b:getPosition()
	if pigx + 2 < self.boundaries.left or pigx - 2 > self.boundaries.right or
	   pigy > self.boundaries.bottom then
		pig:death()
		pig.outside = true
	end

	pig:update(dt)
end

local findGroundCallback
do
	local bx, by
	findGroundCallback = function(fix, x, y, nx, ny, f)
		if not fix then
			return bx, by
		end

		if fix == game.world.terrain.f then
			bx, by = x, y
			return 0
		end

		return -1
	end
end

function world:findGround()
	local px, py = self.pig.b:getPosition()
	self.world:rayCast(px, py, px, 400, findGroundCallback)
	local gx, gy = findGroundCallback()

	if not (gx and gy) then
		return 200, 10, 200, 300, 290
	end
	local diff = gy - py
	return px, py, gx, gy, diff
end

function world:draw()
	local tr, pig = self.terrain, self.pig
	love.graphics.setColor(255, 255, 255, 150)
	love.graphics.draw(self.clouds, -30, -50, 0, game.scale*0.9, game.scale*0.9)
	love.graphics.setColor(255, 255, 255)
	tr:draw()
	pig:draw()
end

function world:reset()
	self.timer = 0
	self.timelanded = nil
	self.beeptimer = 1
	self.pig:reset()
end

return world
